package de.intevation.flys.artifacts.model.fixings;

import org.apache.log4j.Logger;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;
import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.model.CalculationResult;
import de.intevation.flys.artifacts.model.DataFacet;
import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.states.DefaultState.ComputeType;
import de.intevation.flys.utils.KMIndex;

public class FixLongitudinalAvSectorFacet
extends DataFacet
implements FacetTypes {

    /** House logger. */
    private static Logger logger =
        Logger.getLogger(FixLongitudinalAvSectorFacet.class);

    /** Trivial Constructor. */
    public FixLongitudinalAvSectorFacet() {
    }


    public FixLongitudinalAvSectorFacet(
        int ndx,
        String name,
        String description)
    {
        super(
            ndx,
            name,
            description,
            ComputeType.ADVANCE,
            null,
            null);
    }


    /**
     * Returns the data this facet requires.
     *
     * @param artifact the owner artifact.
     * @param context  the CallContext.
     *
     * @return the data as KMIndex.
     */
    @Override
    public Object getData(Artifact artifact, CallContext context) {
        logger.debug("FixLongitudinalAvSectorFacet.getData");

        if (artifact instanceof FLYSArtifact) {
            FLYSArtifact flys = (FLYSArtifact)artifact;

            CalculationResult res =
                (CalculationResult) flys.compute(context,
                                                 ComputeType.ADVANCE,
                                                 false);

            FixAnalysisResult result = (FixAnalysisResult) res.getData();

            KMIndex<AnalysisPeriod []> kmPeriods = result.getAnalysisPeriods();
            if (kmPeriods == null) {
                logger.warn("No analysis periods found.");
                return null;
            }
            int periodNdx = index >> 2;
            KMIndex<AnalysisPeriod> resPeriods =
                    new KMIndex<AnalysisPeriod>();
            for (KMIndex.Entry<AnalysisPeriod[]> entry: kmPeriods) {
                AnalysisPeriod ap = entry.getValue()[periodNdx];
                resPeriods.add(entry.getKm(), ap);
            }

            return resPeriods;
        }
        else {
            logger.warn("Artifact is no instance of FLYSArtifact.");
            return null;
        }
    }


    /**
     * Create a deep copy of this Facet.
     * @return a deep copy.
     */
    @Override
    public FixLongitudinalAvSectorFacet deepCopy() {
        FixLongitudinalAvSectorFacet copy = new FixLongitudinalAvSectorFacet();
        copy.set(this);
        return copy;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
